<?php

namespace Fgc\SecurePay\Block;

use Fgc\SecurePay\Helper\Config;
use Fgc\SecurePay\Helper\QuoteData;
use Magento\Framework\DataObject;
use Magento\Framework\Phrase;
use Magento\Framework\View\Element\Template;
use Magento\Sales\Model\Order;

class DccInvoice extends Template
{
    protected QuoteData $quoteData;
    protected Config $configHelper;
    protected ?array $dccQuote = null;
    protected ?string $area;

    public function __construct(
        Template\Context $context,
        QuoteData $quoteData,
        Config $configHelper,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->quoteData = $quoteData;
        $this->configHelper = $configHelper;
        $this->area = $data['area'] ?? null;
    }

    public function initTotals()
    {
        $fxRateRow = $this->getFxRateRow();
        if ($fxRateRow) {
            $this->getParentBlock()->addTotal(
                $fxRateRow,
                'grand_total'
            );
            $this->getParentBlock()->addTotal(
                $this->getFinalTransactionAmountRow(),
                'securepay_dcc_fx_rate'
            );
            $this->getParentBlock()->addTotal(
                new DataObject([
                    'block_name' => 'securepay_dcc_message'
                ]),
                'securepay_dcc_txn_amount'
            );
        }

        return $this;
    }

    public function getOrder(): Order
    {
        return $this->getParentBlock()->getOrder();
    }

    public function getFxRateRow(): ?DataObject
    {
        $dccQuote = $this->getDccQuote();
        $rate = $dccQuote['converted']['exchangeRate']['value'] ?? null;
        $margin = $dccQuote['converted']['exchangeRate']['markup'] ?? null;
        $currency = $dccQuote['converted']['currency'] ?? null;

        if (isset($rate, $margin, $currency)) {
            $currency = $this->configHelper->toStoreCurrencyCode($currency);
            $value = __('%1 1 = %2 %3', $this->configHelper->toStoreCurrencyCode('AUD'), $currency, $rate);
            return new DataObject(
                [
                    'code' => 'securepay_dcc_fx_rate',
                    'value' => $value,
                    'base_value' => $value,
                    'label' => __('Fx Rate (Including %1% margin)', $margin),
                    'is_formated' => true,
                    'area' => $this->area
                ]
            );
        }

        return null;
    }

    public function getFinalTransactionAmountRow(): ?DataObject
    {
        $dccQuote = $this->getDccQuote();
        $currency = $dccQuote['converted']['currency'] ?? null;
        $amount = $dccQuote['converted']['amount'] ?? null;

        if (isset($amount, $currency)) {
			if (strcasecmp($currency, 'JPY') !== 0) {
				$amount /= 100.0;
			}
            $value = __('%1 %2', $currency, number_format($amount, 2));
            return new DataObject(
                [
                    'code' => 'securepay_dcc_txn_amount',
                    'value' => $value,
                    'base_value' => $value,
                    'label' => __('Final Transaction Amount'),
                    'is_formated' => true,
                    'area' => $this->area,
                    'strong' => true
                ]
            );
        }

        return null;
    }

    public function getDccMessage(): Phrase
    {
        $dccQuote = $this->getDccQuote();
        $currency = $dccQuote['converted']['currency'] ?? null;
        return __(
            'This currency conversion has been provided by %1. I have been offered a choice of currencies for payment including %2 and agree to pay in %3.',
            $this->configHelper->getMerchantName(),
            $this->configHelper->toStoreCurrencyCode('AUD'),
            $this->configHelper->toStoreCurrencyCode($currency),
        );
    }

    protected function getDccQuote()
    {
        if ($this->dccQuote === null) {
            $order = $this->getParentBlock()->getOrder();
            $this->dccQuote = $this->quoteData->getDccQuote($order);
        }

        return $this->dccQuote;
    }
}
