<?php

namespace Fgc\SecurePay\Block\Success;

use Fgc\SecurePay\Helper\Config;
use Fgc\SecurePay\Helper\QuoteData;
use Magento\Checkout\Model\Session;
use Magento\Framework\DataObject;
use Magento\Framework\View\Element\Template;
use Magento\Sales\Model\Order;

class DccInvoice extends \Fgc\SecurePay\Block\DccInvoice
{
    protected Session $checkoutSession;

    public function __construct(
        Template\Context $context,
        QuoteData $quoteData,
        Config $configHelper,
        Session $checkoutSession,
        array $data = []
    ) {
        parent::__construct($context, $quoteData, $configHelper, $data);
        $this->checkoutSession = $checkoutSession;
    }

    public function getOrder(): Order
    {
        return $this->checkoutSession->getLastRealOrder();
    }

    protected function getDccQuote()
    {
        if ($this->dccQuote === null) {
            $order = $this->checkoutSession->getLastRealOrder();
            $this->dccQuote = $this->quoteData->getDccQuote($order);
        }

        return $this->dccQuote;
    }

    public function getTotalRow(): DataObject
    {
        return new DataObject(
            [
                'code' => 'securepay_dcc_original_amount',
                'value' => $this->getOrder()->getGrandTotal(),
                'base_value' => $this->getOrder()->getGrandTotal(),
                'label' => __('Original Transaction Amount'),
                'area' => $this->area
            ]
        );
    }

    /**
     * @return DataObject[]
     */
    public function getRows(): array
    {
        $fxRateRow = $this->getFxRateRow();
        if ($fxRateRow) {
            return [
                $this->getTotalRow(),
                $fxRateRow,
                $this->getFinalTransactionAmountRow()
            ];
        }

        return [];
    }

    public function formatValue($total)
    {
        if (!$total->getIsFormated()) {
            return $this->getOrder()->formatPrice($total->getValue());
        }
        return $total->getValue();
    }

    public function hasReceipt(): bool
    {
        return (bool)$this->getDccQuote();
    }
}
