<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Fgc\SecurePay\Gateway\Request;

use Fgc\SecurePay\Gateway\Constants;
use Fgc\SecurePay\Helper\Config;
use Fgc\SecurePay\Helper\QuoteData;
use InvalidArgumentException;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;

abstract class AbstractInitiationRequest implements BuilderInterface
{
    protected Config $configHelper;
    protected RequestInterface $request;
    protected SerializerInterface $serializer;
    protected QuoteData $quoteData;

    public function __construct(
        Config $configHelper,
        RequestInterface $request,
        SerializerInterface $serializer,
        QuoteData $quoteData
    ) {
        $this->configHelper = $configHelper;
        $this->request = $request;
        $this->serializer = $serializer;
        $this->quoteData = $quoteData;
    }

    /**
     * Builds ENV request
     *
     * @param array $buildSubject
     * @return array
     */
    public function build(array $buildSubject): array
    {
        if (!isset($buildSubject['payment'])
            || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new InvalidArgumentException('Payment data object should be provided');
        }

        /** @var PaymentDataObjectInterface $payment */
        $payment = $buildSubject['payment'];
        $order = $payment->getOrder();

        $requestBody = $this->request->getContent();
        $requestData = $this->serializer->unserialize($requestBody);

        $result = [
            Constants::KEY_ORDER_TYPE => $this->getOrderType(),
            Constants::KEY_ORDER_ID => $order->getOrderIncrementId(),
            Constants::KEY_AMOUNT => $order->getGrandTotalAmount() * 100,
            Constants::KEY_TOKEN => $requestData['paymentMethod']['additional_data']['token'],
            Constants::KEY_CARD_DETAILS => $requestData['paymentMethod']['additional_data'],
            Constants::STORE_IN_VAULT_ON_SUCCESS => $this->configHelper->isSecurePayVaultActive()
        ];

        if ($this->configHelper->isEnableDcc()) {
            $dccQuote = $this->serializer->unserialize($requestData['paymentMethod']['additional_data']['dccQuote']);
            if (isset($dccQuote['converted'])) {
                $result[Constants::KEY_CURRENCY] = $dccQuote['converted']['currency'];
                $result[Constants::KEY_AMOUNT] = $dccQuote['converted']['amount'];
                $this->quoteData->setDccQuote($dccQuote);
            }
        }

        return $result;
    }

    abstract public function getOrderType(): string;
}
