<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Fgc\SecurePay\Gateway\Request;

use Fgc\SecurePay\Gateway\Constants;
use Fgc\SecurePay\Model\Ui\ConfigProvider;
use InvalidArgumentException;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;

class RefundRequest implements BuilderInterface
{
    /**
     * @var ConfigProvider
     */
    private $configProvider;

    /**
     * @param ConfigProvider $configProvider
     */
    public function __construct(
        ConfigProvider $configProvider
    )
    {
        $this->configProvider = $configProvider;
    }

    /**
     * Builds ENV request
     *
     * @param array $buildSubject
     * @return array
     */
    public function build(array $buildSubject)
    {
        if (!isset($buildSubject['payment'])
            || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new InvalidArgumentException('Payment data object should be provided');
        }

        $requestBody = file_get_contents('php://input');
        $requestData = json_decode($requestBody, true);

        /** @var PaymentDataObjectInterface $payment */
        $payment = $buildSubject['payment'];
        $order = $payment->getOrder();

        $amount = $buildSubject['amount'];
        $config = $this->configProvider->getPrivateConfig();

        return array_merge(
            [
                Constants::KEY_ORDER_TYPE => Constants::ORDER_TYPE_REFUND,
                Constants::KEY_ORDER_ID => $order->getOrderIncrementId(),
                Constants::KEY_AMOUNT => $amount * 100,
            ],
            $config
        );
    }
}
