<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Fgc\SecurePay\Gateway\Request;

use Fgc\SecurePay\Gateway\Constants;
use Fgc\SecurePay\Model\Ui\ConfigProvider;
use InvalidArgumentException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;
use Magento\Sales\Model\Order\Payment;

class VaultCaptureRequest implements BuilderInterface
{
    /**
     * @var ConfigProvider
     */
    private $configProvider;

    /**
     * @param ConfigProvider $configProvider
     */
    public function __construct(
        ConfigProvider $configProvider
    )
    {
        $this->configProvider = $configProvider;
    }

    /**
     * Builds ENV request
     *
     * @param array $buildSubject
     * @return array
     * @throws CouldNotSaveException
     */
    public function build(array $buildSubject)
    {
        $errorMessageWithPlaceHolder = 'An error occurred in the transaction process: %1. Please try again.';

        if (!isset($buildSubject['payment'])
            || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new InvalidArgumentException('Payment data object should be provided');
        }

        $config = $this->configProvider->getPrivateConfig();

        /** @var PaymentDataObjectInterface $paymentDO */
        $paymentDO = $buildSubject['payment'];
        $order = $paymentDO->getOrder();
        /** @var Payment $payment */
        $payment = $paymentDO->getPayment();
        $paymentToken = $payment->getExtensionAttributes()->getVaultPaymentToken()->getGatewayToken();

        if ($paymentToken === null) {
            throw new CouldNotSaveException(__($errorMessageWithPlaceHolder, 'The saved payment method is not available to perform the request'));
        }

        return array_merge(
            [
                Constants::KEY_ORDER_TYPE => Constants::ORDER_TYPE_VAULT_CAPTURE,
                Constants::KEY_ORDER_ID => $order->getOrderIncrementId(),
                Constants::KEY_AMOUNT => $order->getGrandTotalAmount() * 100,
                Constants::KEY_TOKEN => $paymentToken,
                Constants::KEY_CUSTOMER_ID => $order->getCustomerId()
            ],
            $config
        );
    }
}
