<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Fgc\SecurePay\Gateway\Request;

use Fgc\SecurePay\Gateway\Constants;
use Fgc\SecurePay\Model\Ui\ConfigProvider;
use InvalidArgumentException;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Request\BuilderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

class WalletRefundRequest implements BuilderInterface
{
    protected OrderRepositoryInterface $orderRepository;

    /**
     * @param ConfigProvider $configProvider
     */
    public function __construct(
        OrderRepositoryInterface $orderRepository
    ) {
        $this->orderRepository = $orderRepository;
    }

    /**
     * Builds ENV request
     *
     * @param array $buildSubject
     * @return array
     */
    public function build(array $buildSubject)
    {
        if (!isset($buildSubject['payment'])
            || !$buildSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new InvalidArgumentException('Payment data object should be provided');
        }

        /** @var PaymentDataObjectInterface $payment */
        $payment = $buildSubject['payment'];
        $orderInfo = $payment->getOrder();
        $order = $this->orderRepository->get($orderInfo->getId());

        $amount = $buildSubject['amount'];

        return [
            Constants::KEY_ORDER_TYPE => Constants::ORDER_TYPE_REFUND,
            Constants::KEY_ORDER_ID => $order->getData('securepay_wallet_txn_id'),
            Constants::KEY_AMOUNT => $amount * 100,
        ];
    }
}
