<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Fgc\SecurePay\Gateway\Response;

use Fgc\SecurePay\Gateway\Constants;
use Fgc\SecurePay\Helper\Config;
use Fgc\SecurePay\Helper\QuoteData;
use InvalidArgumentException;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use Magento\Payment\Gateway\Response\HandlerInterface;
use Magento\Sales\Model\Order\Invoice;

class Handler implements HandlerInterface
{
    protected QuoteData $quoteData;
    protected Config $configHelper;

    public function __construct(QuoteData $quoteData, Config $configHelper)
    {
        $this->quoteData = $quoteData;
        $this->configHelper = $configHelper;
    }

    /**
     * Handles transaction id
     *
     * @param array $handlingSubject
     * @param array $response
     * @return void
     */
    public function handle(array $handlingSubject, array $response)
    {
        if (!isset($handlingSubject['payment'])
            || !$handlingSubject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new InvalidArgumentException('Payment data object should be provided');
        }
        /** @var PaymentDataObjectInterface $paymentDO */
        $paymentDO = $handlingSubject['payment'];
        $payment = $paymentDO->getPayment();
        /** @var $payment Payment */
        if (isset($response[Constants::KEY_TRANSACTION_RESULT]['bankTransactionId'])) {
            $payment->setTransactionId($response[Constants::KEY_TRANSACTION_RESULT]['bankTransactionId']);
        }

        $order = $payment->getOrder();

        if ($this->configHelper->isEnableDcc()) {
            $dccQuote = $this->quoteData->getDccQuote();
            $this->quoteData->setDccOrderId($this->quoteData->getDccOrderId(), $order, false);
            $this->quoteData->setDccQuote($dccQuote, $order, false);

            $invoice = $this->getCurrentInvoice($payment);
            if ($invoice) {
                $currency = $dccQuote['converted']['currency'] ?? null;
                if ($currency) {
                    $currency = $this->configHelper->toStoreCurrencyCode($currency);
                    $invoice->addComment(
                        __(
                            'This currency conversion has been provided by %1. I have been offered a choice of currencies for payment including %2 and agree to pay in %3',
                            $order->getStore()->getName(),
                            $this->configHelper->toStoreCurrencyCode('AUD'),
                            $currency
                        ),
                        false,
                        true
                    );
                }
            }

            $payment->setIsTransactionClosed(true);
        } else {
            $payment->setIsTransactionClosed(false);
        }
    }

    protected function getCurrentInvoice($payment): ?Invoice
    {
        $result = null;
        $order = $payment->getOrder();
        foreach ($order->getInvoiceCollection() as $invoice) {
            if (!$invoice->getId()) {
                $result = $invoice;
                break;
            }
        }
        return $result;
    }
}
