<?php

namespace Fgc\SecurePay\Helper;

use Magento\Checkout\Model\Session;
use Magento\Framework\Serialize\SerializerInterface;
use Throwable;

class QuoteData
{
    protected const KEY = 'securepay_data';
    protected const KEY_DCC_ORDER_ID = 'dcc_order_id';
    protected const KEY_DCC_QUOTE = 'dcc_quote';
    protected const KEY_THREE_D_ORDER_ID = 'three_d_order_id';

    protected Session $checkoutSession;
    protected SerializerInterface $serializer;

    public function __construct(Session $checkoutSession, SerializerInterface $serializer)
    {
        $this->checkoutSession = $checkoutSession;
        $this->serializer = $serializer;
    }

    public function setDccOrderId($id, $quote = null, $save = true)
    {
        $this->setData(self::KEY_DCC_ORDER_ID, $id, $quote, $save);
    }

    public function getDccOrderId($quote = null)
    {
        return $this->getData(self::KEY_DCC_ORDER_ID, $quote);
    }

    public function setDccQuote($dccQuote, $quote = null, $save = true)
    {
        $this->setData(self::KEY_DCC_QUOTE, $dccQuote, $quote, $save);
    }

    public function getDccQuote($quote = null)
    {
        return $this->getData(self::KEY_DCC_QUOTE, $quote);
    }

    public function setThreeDOrderId($id, $quote = null, $save = true)
    {
        $this->setData(self::KEY_THREE_D_ORDER_ID, $id, $quote, $save);
    }

    public function getThreeDOrderId($quote = null)
    {
        return $this->getData(self::KEY_THREE_D_ORDER_ID, $quote);
    }

    protected function getQuote()
    {
        return $this->checkoutSession->getQuote();
    }

    protected function getData($key, $quote = null)
    {
        if (!$quote) {
            $quote = $this->getQuote();
        }
        try {
            $data = $quote->getData(self::KEY);
            $data = $this->serializer->unserialize($data);
        } catch (Throwable $e) {
            $data = [];
        }

        if (isset($data[$key])) {
            return $data[$key];
        }

        return null;
    }

    protected function setData($key, $value, $quote = null, $save = true)
    {
        if (!$quote) {
            $quote = $this->getQuote();
        }
        try {
            $data = $quote->getData(self::KEY);
            $data = $this->serializer->unserialize($data);
        } catch (Throwable $e) {
            $data = [];
        }

        $data[$key] = $value;

        $data = $this->serializer->serialize($data);
        $quote->setData(self::KEY, $data);
        if ($save) {
            $quote->save();
        }

        return null;
    }
}
