<?php

namespace Fgc\SecurePay\Model;

use Fgc\SecurePay\Api\Data\DccDataInterface;
use Fgc\SecurePay\Api\DccInitiatorInterface;
use Fgc\SecurePay\Helper\Config;
use Fgc\SecurePay\Helper\Gateway;
use Fgc\SecurePay\Helper\QuoteData;
use Fgc\SecurePay\Model\Data\DccData;
use Fgc\SecurePay\Model\Data\DccDataFactory;
use Magento\Checkout\Model\Session;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use SecurePayApi\Model\Response\CardPayment\PaymentOrderObject;
use SecurePayApi\Model\Response\Error\ResponseError;
use SecurePayApi\Request\CardPayment\InitiatePaymentOrderRequest;

class DccInitiator implements DccInitiatorInterface
{
    protected DccDataFactory $dccDataFactory;
    protected Config $configHelper;
    protected Session $checkoutSession;
    protected Gateway $gateway;
    protected RemoteAddress $remoteAddress;
    protected QuoteData $quoteData;

    public function __construct(
        DccDataFactory $dccDataFactory,
        Config $configHelper,
        Session $checkoutSession,
        Gateway $gateway,
        RemoteAddress $remoteAddress,
        QuoteData $quoteData
    ) {
        $this->dccDataFactory = $dccDataFactory;
        $this->configHelper = $configHelper;
        $this->checkoutSession = $checkoutSession;
        $this->gateway = $gateway;
        $this->remoteAddress = $remoteAddress;
        $this->quoteData = $quoteData;
    }

    public function execute(): DccDataInterface
    {
        if (!$this->configHelper->isEnableDcc()) {
            throw new LocalizedException(
                __('Dynamic Currency Conversion is not enabled for this store.')
            );
        }

        $amount = $this->checkoutSession->getQuote()->getGrandTotal() * 100;

        $request = new InitiatePaymentOrderRequest($this->gateway->isLive(), $this->gateway->getCredential(), [
            'merchantCode' => $this->gateway->getCredential()->getMerchantCode(),
            'amount' => $amount,
            'ip' => $this->remoteAddress->getRemoteAddress(),
            'orderType' => 'DYNAMIC_CURRENCY_CONVERSION',
            'metadata' => $this->gateway->getMetadata(),
        ]);

        /** @var PaymentOrderObject $response */
        try {
            $response = $request->execute();
        } catch (\Exception $e) {
            throw new LocalizedException(
                __('An error occurred while attempting this transaction, please try again or contact support.')
            );
        }

        if ($response instanceof ResponseError) {
            throw new LocalizedException(
                __('An error occurred while attempting this transaction, please try again or contact support.')
            );
        }

        /** @var DccData $result */
        $result = $this->dccDataFactory->create();
        $result->setOrderToken($response->getOrderToken());

        $this->quoteData->setDccOrderId($response->getOrderId());

        return $result;
    }
}
