<?php

namespace Fgc\SecurePay\Model\Ui;

use Fgc\SecurePay\Gateway\Constants;
use Fgc\SecurePay\Helper\Config;
use Magento\Checkout\Model\ConfigProviderInterface;
use SecurePayApi\Endpoint;

class ConfigProvider implements ConfigProviderInterface
{

    public const CODE = 'securepay';
    public const VAULT_CODE = 'securepay_vault';

    protected Config $configHelper;

    public function __construct(
        Config $configHelper
    ) {
        $this->configHelper = $configHelper;
    }

    /**
     * Retrieve assoc array of checkout configuration
     *
     * @return array
     */
    public function getConfig(): array
    {
        return [
            'payment' => [
                self::CODE => [
                    'uiInfo' => [
                        'merchantCode' => $this->configHelper->getMerchantCode(),
                        'clientId' => $this->configHelper->getClientId(),
                        'allowedCardTypes' => $this->configHelper->getSecurePayAllowedCardTypes(),
                        'scriptUrl' => $this->configHelper->isSecurePayLive() ?
                            Endpoint::URL_LIVE_SCRIPT : Endpoint::URL_SANDBOX_SCRIPT,
                        'enableThreeD' => $this->configHelper->isEnableThreeD(),
                        'enableDcc' => $this->configHelper->isEnableDcc(),
                        'threeDScriptUrl' => $this->configHelper->isSecurePayLive() ?
                            Endpoint::URL_LIVE_3DS2_SCRIPT : Endpoint::URL_SANDBOX_3DS2_SCRIPT,
                        'enableFraudGuard' => $this->configHelper->isEnableFraudGuard(),
                        'logoUrl' => $this->configHelper->getLogoUrl(),
                    ],
                    'ccVaultCode' => self::VAULT_CODE,
                    'isActive' => $this->configHelper->isSecurePayActive()
                ],
            ]
        ];
    }

    public function getPrivateConfig(): array
    {
        return [
            Constants::KEY_MERCHANT_CODE => $this->configHelper->getMerchantCode(),
            Constants::KEY_CLIENT_ID => $this->configHelper->getClientId(),
            Constants::KEY_CLIENT_SECRET => $this->configHelper->getClientSecret(),
            Constants::KEY_STORE_ID => $this->configHelper->getStoreUid(),
            Constants::STORE_IN_VAULT_ON_SUCCESS => $this->configHelper->isSecurePayVaultActive(),
        ];
    }
}
