<?php

namespace Fgc\SecurePay\Observer;

use Fgc\SecurePay\Helper\Config;
use Magento\Config\Model\ResourceModel\Config as ResourceConfig;
use Magento\Framework\App\Config\ReinitableConfigInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Store\Model\ResourceModel\Store\CollectionFactory as StoreCollectionFactory;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\Store;

class PaymentConfigSaveAfter implements ObserverInterface
{
    protected ResourceConfig $resourceConfig;
    protected ReinitableConfigInterface $reinitableConfig;
    protected StoreCollectionFactory $storeCollectionFactory;

    public function __construct(
        ResourceConfig $resourceConfig,
        ReinitableConfigInterface $reinitableConfig,
        StoreCollectionFactory $storeCollectionFactory
    ) {
        $this->resourceConfig = $resourceConfig;
        $this->reinitableConfig = $reinitableConfig;
        $this->storeCollectionFactory = $storeCollectionFactory;
    }

    public function execute(Observer $observer)
    {
        $changedPaths = (array)$observer->getEvent()->getChangedPaths();
        $needReset = false;
        foreach ($changedPaths as $changedPath) {
            if (stripos($changedPath, Config::GROUP_SECURE_PAY)) {
                $needReset = true;
                break;
            }
        }
        if ($needReset) {
            $tokenPath = Config::BASE_CONFIG_PATH . '/' . Config::GROUP_SECURE_PAY . '/token';
            /** @var Store $store */
            foreach ($this->storeCollectionFactory->create() as $store) {
                $this->resourceConfig->deleteConfig(
                    $tokenPath,
                    ScopeInterface::SCOPE_STORES,
                    $store->getId()
                );
            }

            $this->reinitableConfig->reinit();
        }
    }
}
