/*browser:true*/
/*global define*/
define([
    'jquery',
    'underscore',
    'Magento_Checkout/js/model/quote',
    'mage/storage',
    'Magento_Checkout/js/model/error-processor',
    'Magento_Checkout/js/model/url-builder'
], function ($, _, quote, storage, errorProcessor, urlBuilder) {
    'use strict';
    var securePayThreedsUI;

    function prepareInputData(cardToken, cardholderName, overrides = {}) {
        var quoteBillingAddress = quote.billingAddress();
        var quoteShippingAddress = quote.shippingAddress();
        var billingAddress = {
            city: quoteBillingAddress.city,
            state: quoteBillingAddress.regionCode,
            country: quoteBillingAddress.countryId,
            zipCode: quoteBillingAddress.postcode,
            streetAddress: quoteBillingAddress.street[0],
            detailedStreetAddress: quoteBillingAddress.street[1],
            detailedStreetAddressAdditional: quoteBillingAddress.street[2]
        };
        var shippingAddress = {
            city: quoteShippingAddress.city,
            state: quoteShippingAddress.regionCode,
            country: quoteShippingAddress.countryId,
            zipCode: quoteShippingAddress.postcode,
            streetAddress: quoteShippingAddress.street[0],
            detailedStreetAddress: quoteShippingAddress.street[1],
            detailedStreetAddressAdditional: quoteShippingAddress.street[2]
        };
        var shippingMethodType = quote.isVirtual() ? '07' : '00';
        if (shippingMethodType === '00') {
            shippingMethodType = _.isEqual(shippingAddress, billingAddress) ? '01' : '03';
        }
        var customerEmail = window.isCustomerLoggedIn ? window.customerData.email : quote.guestEmail;

        var result = {
            cardTokenInfo: {
                cardholderName: cardholderName,
                cardToken: cardToken
            },
            accountData: {
                emailAddress: customerEmail,
            },
            billingAddress: billingAddress,
            shippingAddress: shippingAddress,
            merchantRiskData: {
                deliveryTimeframeType: quote.isVirtual() ? '04' : '01',
                reOrderType: '01',
                shippingMethodType: shippingMethodType,
                deliveryEmailAddress: shippingMethodType === '05' ? customerEmail : undefined,
            },
            threeDSInfo: {
                threeDSReqAuthMethodInd: '02'
            }
        };

        return $.extend(true, result, overrides);
    }

    return function (cardToken, cardholderName, iframeElement, onThreeDSResultsResponse, onThreeDSError) {
        if (!securePayThreedsUI) {
            securePayThreedsUI = new window.SecurePayThreedsUI();
        }

        storage.post(
            urlBuilder.createUrl('/securepay/initiate-3ds2', {})
        ).done(function (response) {
            securePayThreedsUI.initAndStartThreeDS({
                clientId: response.provider_client_id,
                iframe: iframeElement,
                token: response.order_token,
                simpleToken: response.simple_token,
                threeDSSessionId: response.session_id,
                onRequestInputData: function () {
                    return prepareInputData(cardToken, cardholderName, {
                        merchantRiskData: JSON.parse(response.risk_data || '{}')
                    });
                },
                onThreeDSResultsResponse: onThreeDSResultsResponse,
                onThreeDSError: onThreeDSError
            });
        }).fail(function (response) {
            onThreeDSError && onThreeDSError();
        })
    }
})
