define([
    'Magento_Checkout/js/view/payment/default',
    'jquery',
    'ko',
    'underscore',
    'Magento_Checkout/js/model/quote',
    'Magento_Checkout/js/model/url-builder',
    'mage/storage',
    'mage/translate',
    'Magento_Checkout/js/model/error-processor',
    'Magento_Checkout/js/model/full-screen-loader',
    'Magento_Checkout/js/model/payment/additional-validators',
    'Magento_Checkout/js/action/redirect-on-success'
], function (Component, $, ko, _, quote, urlBuilder, storage, $t, errorProcessor, fullScreenLoader, additionalValidators, redirectOnSuccessAction) {
    'use strict';

    return Component.extend({
        defaults: {
            template: 'Fgc_SecurePay/payment/applepay-form',
        },

        paymentData: null,
        logoUrl: window.checkoutConfig.payment.securepay_applepay.logoUrl,

        scanAndPlaceOrder: function () {
            var self = this;

            fullScreenLoader.startLoader();
            var config = window.checkoutConfig.payment.securepay_applepay;

            const session = new ApplePaySession(3, {
                countryCode: config.countryCode,
                currencyCode: config.currencyCode,
                merchantCapabilities: ['supports3DS'],
                supportedNetworks: config.cardTypes,
                total: {
                    amount: quote.totals().grand_total,
                    label: config.storeName,
                    type: 'final'
                }
            })
            session.onvalidatemerchant = function (event) {
                storage.post(
                    urlBuilder.createUrl('/securepay/initiate-applepay', {})
                ).done(function (response) {
                    session.completeMerchantValidation(JSON.parse(response.session_object));
                }).fail(function (response) {
                    session.abort();
                    errorProcessor.process(response, self.messageContainer);
                    fullScreenLoader.stopLoader();
                });
            }

            session.onpaymentauthorized = function (event) {
                self.paymentData = event.payment.token.paymentData;
                self.placeOrder.bind(self)(session);
            }

            session.begin();
        },

        getId: function () {
            return 'securepay_applepay'
        },

        getData: function () {
            let data = {
                'method': this.item.method,
                'additional_data': {}
            };

            if (this.paymentData) {
                data.additional_data = {
                    paymentData: JSON.stringify({
                        encryptedData: this.paymentData.data,
                        signature: this.paymentData.signature,
                        version: this.paymentData.version,
                        ephemeralPublicKey: this.paymentData.header.ephemeralPublicKey,
                        publicKeyHash: this.paymentData.header.publicKeyHash,
                        transactionId: this.paymentData.header.transactionId,
                        applicationData: this.paymentData.header.applicationData,
                    })
                }
            }

            return data;
        },

        isButtonActive: function () {
            return this.isActive() && this.isPlaceOrderActionAllowed();
        },

        isActive: function () {
            return this.isChecked() === this.getId();
        },

        /**
         * Place order.
         */
        placeOrder: function (session) {
            var self = this;

            if (this.validate() &&
                additionalValidators.validate() &&
                this.isPlaceOrderActionAllowed() === true
            ) {
                this.isPlaceOrderActionAllowed(false);

                this.getPlaceOrderDeferredObject()
                    .done(function () {
                        session.completePayment(session.STATUS_SUCCESS);
                        redirectOnSuccessAction.execute();
                    })
                    .fail(function () {
                        session.completePayment(session.STATUS_FAILURE);
                    })
                    .always(function () {
                        self.isPlaceOrderActionAllowed(true);
                        fullScreenLoader.stopLoader();
                    });

                return true;
            }

            fullScreenLoader.stopLoader();
            return false;
        },
    })
})
