/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
/*browser:true*/
/*global define*/
define(
    [
        'Magento_Checkout/js/view/payment/default',
        'jquery',
        'ko',
        'underscore',
        'Magento_Checkout/js/model/quote',
        'mage/translate',
        'Magento_Vault/js/view/payment/vault-enabler',
        'Magento_Checkout/js/model/full-screen-loader',
        'Fgc_SecurePay/js/action/three-d',
        'Fgc_SecurePay/js/action/dcc-init',
        'Magento_Ui/js/modal/modal'
    ],
    function (Component, $, ko, _, quote, $t, VaultEnabler, fullScreenLoader, threeDAction, dccInitAction, modal) {
        'use strict';

        const uiInfo = window.checkoutConfig.payment.securepay.uiInfo;

        return Component.extend({
            defaults: {
                template: 'Fgc_SecurePay/payment/form'
            },
            transactionResult: '',
            cardHolder: ko.observable(''),
            cardHolderError: ko.observable(false),
            showForm: ko.observable(true),
            grandTotal: ko.observable(0),
            logoUrl: uiInfo.logoUrl,

            initialize: function () {
                var self = this;

                self._super();
                self.vaultEnabler = new VaultEnabler();
                self.vaultEnabler.setPaymentCode(this.getVaultCode());
                self.cardHolder.subscribe(function (cardHolder) {
                    if (!cardHolder) {
                        self.cardHolderError($t('This is a required field.'));
                    } else {
                        self.cardHolderError(false);
                    }
                })

                self.grandTotal(quote.totals().grand_total);
                quote.totals.subscribe(function (totals) {
                    self.grandTotal(totals.grand_total);
                })

                return self;
            },

            initSecurePayUi: _.once(function () {
                var self = this;
                self.tokenisedCard = null;
                $('#securepay-ui-script-container').append(`<script id="${self.getScriptId()}" src="${self.getScriptUrl()}"></script>`)
                if (self.isEnableThreeD()) {
                    $('#securepay-ui-script-container').append(`<script id="sp-threeds-js" src="${uiInfo.threeDScriptUrl}" type="text/javascript"></script>`);
                }

                const init = function () {
                    self.showForm(false);
                    self.showForm(true);

                    fullScreenLoader.startLoader();
                    var securepayOptions = {
                        containerId: self.getUiContainer(),
                        scriptId: self.getScriptId(),
                        clientId: uiInfo.clientId,
                        merchantCode: uiInfo.merchantCode,
                        card: {
                            allowedCardTypes: uiInfo.allowedCardTypes,
                            showCardIcons: true,
                            onTokeniseSuccess: function (tokenisedCard) {
                                self.tokenisedCard = tokenisedCard;
                                self.process();
                            },
                            onTokeniseError: function (errors) {
                                fullScreenLoader.stopLoader();
                                self.messageContainer.addErrorMessage({message: $t('Error occurs in tokenisation process, please try again.')});
                            },
                            onCardTypeChange: function(cardType) {
                                self.tokenisedCard = null;
                            },
                            onBINChange: function(cardBIN) {
                                self.tokenisedCard = null;
                            },
                            onFormValidityChange: function(valid) {
                                if (!valid) {
                                    self.tokenisedCard = null;
                                }
                            },
                        },
                        onLoadComplete: function () {
                            var uiContainer = $('#' + self.getUiContainer());
                            uiContainer.find('iframe').width(0);
                            uiContainer.find('iframe').width(Math.floor(uiContainer.width()));

                            $(window).on('resize', function () {
                                uiContainer.find('iframe').width(0);
                                uiContainer.find('iframe').width(Math.floor(uiContainer.width()));
                            })
                            fullScreenLoader.stopLoader();
                        },
                        metadata: {
                            shoppingCartPlatform: {
                                version: "2.4.3",
                                name: "Magento"
                            }
                        }
                    };

                    if (self.isEnableDcc()) {
                        securepayOptions.mode = 'dcc';
                        securepayOptions.card.onDCCQuoteSuccess = function () {
                            console.log('onDCCQuoteSuccess', arguments);
                        }
                        securepayOptions.card.onDCCQuoteError = function () {
                            console.log('onDCCQuoteError', arguments);
                        }

                        dccInitAction().done(function (response) {
                            securepayOptions.checkoutInfo = {orderToken: response.order_token};
                            self.mySecurePayUI = new securePayUI.init(securepayOptions);
                        })
                    } else {
                        self.mySecurePayUI = new securePayUI.init(securepayOptions);
                    }
                };

                const interval = setInterval(function () {
                    if ($('#' + self.getUiContainer() + ':visible').length && typeof window.securePayUI !== 'undefined') {
                        init();
                        clearInterval(interval);
                    }
                }, 100);

                if (self.isEnableDcc()) {
                    self.grandTotal.subscribe(function () {
                        const interval2 = setInterval(function () {
                            if ($('#' + self.getUiContainer() + ':visible').length && typeof window.securePayUI !== 'undefined') {
                                init();
                                clearInterval(interval2);
                            }
                        }, 100);
                    })
                }
            }),

            getScriptId: function () {
                return 'securepay-ui-js';
            },

            getScriptUrl: function () {
                return uiInfo.scriptUrl;
            },

            getUiContainer: function () {
                return 'securepay-ui-container';
            },

            getCode: function () {
                return 'securepay';
            },

            isEnableThreeD: function () {
                return !uiInfo.enableDcc && uiInfo.enableThreeD;
            },

            isEnableDcc: function () {
                return !!uiInfo.enableDcc;
            },

            getData: function () {
                let data = {
                    'method': this.item.method,
                    'additional_data': {}
                };

                if (this.tokenisedCard) {
                    data.additional_data = {
                        'token': this.tokenisedCard.token
                    }

                    if (this.isEnableDcc()) {
                        data.additional_data.dccQuote = JSON.stringify(this.tokenisedCard.dccQuote);
                    }
                }

                data['additional_data'] = _.extend(data['additional_data'], this.additionalData);
                this.vaultEnabler.visitAdditionalData(data);

                return data;
            },

            isVaultEnabled: function () {
                return this.vaultEnabler.isVaultEnabled();
            },

            getVaultCode: function () {
                return window.checkoutConfig.payment[this.getCode()].ccVaultCode;
            },

            process: function () {
                var self = this;
                if (self.isEnableThreeD()) {
                    let iframeId = '3ds-v2-challenge-iframe';
                    var iframe = $(`<iframe id="${iframeId}" name="${iframeId}" style="width: 100%; height: 500px; visibility: hidden"></iframe>`);
                    var wrapper = $('#3ds2-iframe-wrapper');
                    wrapper.children().remove();
                    wrapper.append(iframe);
                    threeDAction(
                        self.tokenisedCard.token,
                        self.cardHolder(),
                        document.getElementById(iframeId),
                        function (response) {
                            fullScreenLoader.stopLoader();

                            if (response.liabilityShiftIndicator === 'Y') {
                                self.placeOrder();
                            } else {
                                let message = 'Unable to authenticate';
                                switch (response.transStatus) {
                                    case 'A':
                                        message = 'Attempted Authentication';
                                        break;
                                    case 'N':
                                        message = '	Not Authenticated';
                                        break;
                                    case 'U':
                                        message = 'Unable to authenticate';
                                        break;
                                    case 'R':
                                        message = 'Rejected';
                                        break;
                                    default:
                                        message = 'Unable to authenticate';
                                        break;
                                }
                                self.messageContainer.addErrorMessage({
                                    message: $t('Error occurs in 3DS2 validation process: %1. Please try again.').replace('%1', message)
                                });
                            }
                        },
                        function (response) {
                            var error;
                            if (response && response.errors && response.errors[0]) {
                                error = response.errors[0].detail;
                            }

                            fullScreenLoader.stopLoader();
                            if (error) {
                                self.messageContainer.addErrorMessage({
                                    message: $t('Error occurs in 3DS2 validation process: %1. Please try again.').replace('%1', error)
                                });
                            } else {
                                self.messageContainer.addErrorMessage({message: $t('Error occurs in 3DS2 validation process, please try again.')});
                            }
                        });
                } else {
                    fullScreenLoader.stopLoader();
                    self.placeOrder();
                }
            },

            tokeniseAndPlaceOrder: function () {
                if (this.isEnableThreeD()) {
                    if (!this.cardHolder()) {
                        this.cardHolderError($t('This is a required field.'));
                        return false;
                    } else {
                        this.cardHolderError(false);
                    }
                }
                if (typeof this.mySecurePayUI !== 'undefined') {
                    fullScreenLoader.startLoader();
                    this.mySecurePayUI.tokenise();
                }
                return false;
            }
        });
    }
);
